function Game() {
    this.$recordsPopup = $("#records_popup");
    this.$popupPointsParagraph = $("#points_paragraph");
    this.$popupPlayAgainBtn = $("#records_popup_btn");
    this.$popupPlayAgainBtn.click(function(e) {
        objects.game.$recordsPopup.addClass(domElements.invisibleClassName);
        objects.game.startNewGame();
        utils.turnOnScreen(domElements.$gameScreen);
    });

    this.$pointsSpan = $("#points_span");
    this.$livesSpan = $("#lives_span");

    this.newPoint = undefined;
    this.tick = this.timer = this.clearTimeout = this.pauseTimeout = undefined; //RESIZE
    this.background = undefined;
    this.weapon = this.screen = undefined;
    this.points = this.lives = undefined;
    this.target = this.count = this.countTimer = undefined;

    this.setView();
}

Game.prototype.setView = function() {
    this.background = new paper.Path.Rectangle(new paper.Point(0, 0), new paper.Size(constants.GAME_WIDTH, constants.GAME_HEIGHT));
    this.background.fillColor = settings.BACKGROUND_COLOR;
    this.screen = new Screen();
    this.target = new Target();
    this.weapon = new Weapon();
    this.count = new Count();
    this.setTargetListeners();
}

Game.prototype.setTargetListeners = function() {
    var game = this;
    $(this.target).bind(events.TARGET_ESCAPED, function(e) {
        game.lives--;
        game.setTexts();
        if(game.lives <= 0) {
            game.endGame();
            return;
        }
        game.startNewLevel();
    });
    $(this.target).bind(events.TARGET_MISSED, function(e) {
        game.target.setNextLevelParameters();
        game.startNewLevel();
    });
    $(this.target).bind(events.TARGET_SHOT, function(e) {
        game.points += game.target.points;
        game.setTexts();
    });
}

Game.prototype.setPointView = function(clickPoint) {
    this.newPoint = clickPoint;
}

Game.prototype.setShot = function(clickPoint) {
    if(typeof (this.weapon.shotTimer) != "undefined" || typeof (this.countTimer) != "undefined") return;
    this.weapon.shoot();
    this.target.checkShot(clickPoint);
}

Game.prototype.startNewGame = function() {
    this.newPoint = undefined;
    this.setPointView(new paper.Point(0, 0));
    this.lives = settings.START_LIVES;
    this.points = 0;
    this.setTexts();
    this.target.setStartParameters();

    this.tick = 0;
    this.timer = window.setInterval(function() {
        objects.game.onFrame();
    }, constants.MILISECONDS_FOR_FRAME);

    this.startNewLevel();
}

Game.prototype.startNewLevel = function() {
    this.countTimer = settings.SECONDS_BEFORE_SHOT * 1000 + settings.GO_MILISECONDS;
}

// RESIZE
Game.prototype.pauseGame = function(pauseMilliseconds) {
    if(typeof (this.timer) == "undefined") return;
    window.clearInterval(this.timer);
    this.timer = undefined;
    this.pauseTimeout = window.setTimeout(function(){objects.game.resumeGame();}, pauseMilliseconds);
}

Game.prototype.resumeGame = function() {
    window.clearTimeout(this.pauseTimeout);
    this.pauseTimeout = undefined;
    this.timer = window.setInterval(function() {objects.game.onFrame();}, constants.MILISECONDS_FOR_FRAME);
}
//

Game.prototype.onFrame = function() {
    if(typeof(this.newPoint) != "undefined") {
        this.weapon.setPointView(this.newPoint);
        this.screen.setParalax(this.newPoint);
        this.target.setParalax(this.newPoint);
        this.newPoint = undefined;
    }
    this.tick += constants.MILISECONDS_FOR_FRAME;
	this.weapon.onFrame();
    this.target.onFrame();
    this.screen.onFrame();
    if(typeof (this.countTimer) != "undefined") {
        this.setCountBeforeShot();
    }

	paper.view.draw();
}

Game.prototype.setCountBeforeShot = function() {
    if((this.countTimer - settings.GO_MILISECONDS) % 1000 == 0) {
        this.count.setNewText((this.countTimer - settings.GO_MILISECONDS) / 1000);
    }
    if(this.countTimer == settings.GO_MILISECONDS) {
        this.count.setNewText("GO");
    }
    this.countTimer -= constants.MILISECONDS_FOR_FRAME;
    this.count.onFrame();
    if(this.countTimer <= 0) {
        this.countTimer = undefined;
        this.count.clear();
        this.target.born();
    }
}

Game.prototype.endGame = function() {
    window.clearInterval(this.timer);
    this.timer = undefined;
    this.clearTimer = window.setTimeout(function() {
        objects.game.removeGameObjects();
    }, constants.MILISECONDS_FOR_FRAME * 2);
}

Game.prototype.removeGameObjects = function() {
    window.clearTimeout(this.clearTimer);
    this.clearTimer = undefined;

    variables.currentScore = this.points;
    this.target.clear();

    if(settings.RECORDS_ENABLED) {
        objects.records.setNewRecords();
        utils.turnOnScreen(domElements.$recordScreen);
    }else {
        this.$recordsPopup.removeClass(domElements.invisibleClassName);
        this.$popupPointsParagraph.text(this.points);
        //RECORDS
    }
}

Game.prototype.setTexts = function() {
    this.$pointsSpan.text(this.points);
    this.$livesSpan.text(this.lives);
}